/* dpsftplib.h
    Declarations for dpsftplib.
    Copyright (C) 1999 Eric Gillespie, Jr.

    This file is part of dpsftplib.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2,  as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef dpsftplib_h
#define dpsftplib_h

#include <glib.h>
#include <stdio.h>

#ifdef HAVE_PTHREADS
#include <pthread.h>
#endif

extern const int dpsftplib_major_version;
extern const int dpsftplib_minor_version;
extern const int dpsftplib_micro_version;

/*****************************************************************************/
/*  dpsftplib types */

typedef struct ftpsite      ftpSite;
typedef struct ftpdirectory ftpDirectory;
typedef struct ftpfile      ftpFile;
typedef struct ftptransfer  ftpTransfer;
typedef struct ftpqueueitem ftpQueueItem;
typedef struct ftpqueuedata ftpQueueData;

typedef enum { UP = 1, DOWN } ftpDirection;
typedef enum { HTTP = 1, FTP } ftpProtocol;

/* See prototype for ftpQueueDataGetStatus for an explanation. */
typedef enum {
	FTP_QSTAT_BEGINNING,
	FTP_QSTAT_TRANSFERRING,
	FTP_QSTAT_COMPLETED,
	FTP_QSTAT_ABORTED,
	FTP_QSTAT_CONNECT_RETRY,
	FTP_QSTAT_TRANSFER_RETRY
} ftpQueueStatus;

/* callbacks */
typedef void (*ftpQueueCallback) (const ftpQueueData *data, gpointer userdata);
typedef void (*ftpLogCallback)   (gpointer type, gchar *format, ...);

#define FTP_QUEUE_CALLBACK(cb) ((ftpQueueCallback)cb)
#define FTP_LOG_CALLBACK(cb)   ((ftpLogCallback)cb)

/* A parsed URI; returned by HTScan() */
typedef struct _HTURI {
	char *access;
	char *host;
	char *absolute;
	char *relative;
	char *fragment;
} HTURI;

/*****************************************************************************/
/* Type manipulation functions. */

/* the New functions return a pointer to a new struct, or NULL on fail */
extern ftpSite *ftpSiteNew(gchar *hostname, gint port, gchar *login,
			   gchar *pass, ftpLogCallback logfunc,
			   gpointer command, gpointer reply, gpointer logdata,
			   gint type);
extern ftpDirectory *ftpDirectoryNew(const gchar *name);
extern ftpFile *ftpFileNew(const gchar *name, const gchar *dir, guint32 size,
			   const gchar *date, gint type, const gchar *mode,
			   const gchar *user, const gchar *group);

/* the Delete functions release all memory allocated for each struct */
extern void ftpSiteDelete(ftpSite *site);
extern void ftpFileDelete(ftpFile *file);
extern void ftpDirectoryDelete(ftpDirectory *directory);

/* This convenience function returns a copy of site */
extern ftpSite *ftpSiteCopy(ftpSite *site);

/* accessor functions for the data types */
extern void           ftpSiteSetLogCallback(ftpSite *site,
					    ftpLogCallback logfunc);
extern ftpLogCallback ftpSiteGetLogCallback(const ftpSite *site);
extern void           ftpSiteSetLogCommandData(ftpSite *site, gpointer data);
extern gpointer       ftpSiteGetLogCommandData(const ftpSite *site);
extern void           ftpSiteSetLogReplyData(ftpSite *site, gpointer data);
extern gpointer       ftpSiteGetLogReplyData(const ftpSite *site);
extern void           ftpSiteSetLogData(ftpSite *site, gpointer data);
extern gpointer       ftpSiteGetLogData(const ftpSite *site);
extern void           ftpSiteSetLog(ftpSite *site, ftpLogCallback logfunc,
				    gpointer command, gpointer reply,
				    gpointer logdata);
extern gint           ftpSiteGetType(const ftpSite *site);
extern void           ftpSiteSetType(ftpSite *site, gint type);
extern gboolean       ftpSiteConnected(const ftpSite *site);
extern const gchar   *ftpSiteGetHostname(const ftpSite *site);
extern void           ftpSiteSetHostname(ftpSite *site, gchar *hostname);
extern gint           ftpSiteGetPort(const ftpSite *site);
extern void           ftpSiteSetPort(ftpSite *site, gint port);
extern const gchar   *ftpSiteGetLogin(const ftpSite *site);
extern void           ftpSiteSetLogin(ftpSite *site, gchar *login);
extern const gchar   *ftpSiteGetPass(const ftpSite *site);
extern void           ftpSiteSetPass(ftpSite *site, gchar *pass);
extern char           ftpSiteGetTransferMode(const ftpSite *site);
extern const gchar   *ftpSiteGetReplyBuffer(const ftpSite *site);
extern const gchar   *ftpSiteGetPwd(const ftpSite *site);
extern gboolean       ftpSiteTransferring(const ftpSite *site);
extern gint           ftpSiteLock(ftpSite *site);
extern gint           ftpSiteUnlock(ftpSite *site);

extern gint         ftpFileGetType(const ftpFile *file);
extern void         ftpFileSetType(ftpFile *file, gint type);
extern const gchar *ftpFileGetName(const ftpFile *file);
extern const gchar *ftpFileGetDir(const ftpFile *file);
extern guint32      ftpFileGetSize(const ftpFile *file);
extern const gchar *ftpFileGetMode(const ftpFile *file);
extern const gchar *ftpFileGetUser(const ftpFile *file);
extern const gchar *ftpFileGetGroup(const ftpFile *file);
extern const gchar *ftpFileGetDate(const ftpFile *file);

extern const gchar *ftpDirectoryGetName(const ftpDirectory *directory);
extern const GList *ftpDirectoryGetDirList(const ftpDirectory *dir);
extern const GList *ftpDirectoryGetFileList(const ftpDirectory *dir);

extern const ftpSite      *ftpQueueItemGetSite(const ftpQueueItem *item);
extern void                ftpQueueItemSetSite(ftpQueueItem *item,
					       ftpSite *site);
extern const ftpFile      *ftpQueueItemGetRemoteFile(const ftpQueueItem *item);
extern void                ftpQueueItemSetRemoteFile(ftpQueueItem *item,
						     ftpFile *file);
extern const ftpFile      *ftpQueueItemGetLocalFile(const ftpQueueItem *item);
extern void                ftpQueueItemSetLocalFile(ftpQueueItem *item,
						    ftpFile *file);
extern guint32             ftpQueueItemGetResume(const ftpQueueItem *item);
extern void                ftpQueueItemSetResume(ftpQueueItem *item,
						 guint32 resume);
extern ftpDirection        ftpQueueItemGetDirection(const ftpQueueItem *item);
extern const ftpQueueItem *ftpQueueDataGetItem(const ftpQueueData *data);
extern guint32             ftpQueueDataGetSize(const ftpQueueData *data);
extern guint32             ftpQueueDataGetTransferred(const ftpQueueData*data);

/*
** The next function returns the status of the queue.
** The possible values are:
**  FTP_QSTAT_BEGINNING
**     Transfer of this item is about to begin.
**  FTP_QSTAT_TRANSFERRING
**     Status normal.
**  FTP_QSTAT_COMPLETED
**     Transfer of this item is complete. If the library user is
**     maintaining any sort of display of the queue, this item
**     should now be removed from it.
**  FTP_QSTAT_ABORTED
**     Transfer of this item has been aborted. If the library user is
**     maintaining any sort of display of the queue, this item should
**     now be updated with the new remaining size.
**  FTP_QSTAT_CONNECT_RETRY
**     The connection failed, and is being retried.
**  FTP_QSTAT_TRANSFER_RETRY
**     The transfer failed, and is being retried.
*/
extern ftpQueueStatus ftpQueueDataGetStatus(const ftpQueueData *data);

/*****************************************************************************/
/* Library initialization and shutdown functions. */

/* You must call this function in each thread (excpet main) before it
   accesses any library function, except ftpInit. It sets up ftperrno. */
#ifdef HAVE_PTHREADS
extern void ftpThreadInit(void);
#endif

/* Pass the name of the config file to ftpInit(). It will load the queue
   from the queue file specified by the config file or a default.
   ftpShutdown() will write the queue to the same file */
extern gboolean ftpInit(gchar *config);
extern gboolean ftpShutdown(void);

extern gboolean ftpLoadConf(gchar *fn);
extern gboolean ftpSaveConf(gchar *fn);

/*****************************************************************************/
/* Library configuration functions */

extern gboolean ftpConfSetAnonPass(gchar *val);
extern gboolean ftpConfSetQueueFile(gchar *val);
extern void     ftpConfSetReuseConnection(gboolean val);
extern void     ftpConfSetQueuebytes(gint val);

/*****************************************************************************/
/* Misc FTP functions. */

/*
** ftpCommand() can be used to bypass the library and send a command directly
** to the ftp site. Don't do it unless you know what you're doing! You can
** seriously screw up the library!
**
** ftpGetReply() should only be used on commands you've issued with
** ftpCommand(). Any other use can break things!
*/
extern gint ftpCommand(const ftpSite *site, gchar *format, ...);
extern const gchar *ftpGetReply(ftpSite *site);

extern gboolean ftpOpen(ftpSite *site);
extern gboolean ftpOpenData(ftpSite *site, gchar *mode);
extern gboolean ftpClose(ftpSite *site);
extern gboolean ftpChdir(ftpSite *site, const gchar *dir);
extern gboolean ftpCdup(ftpSite *site);
extern gboolean ftpPwd(ftpSite *site);
extern gboolean ftpType(ftpSite *site, const gint type);
extern gboolean ftpNoop(ftpSite *site);

/* If dir == NULL the current directory is used. Returns NULL on fail. */
extern ftpDirectory *ftpGetDirRemote(ftpSite *site, const gchar *dir);
extern ftpDirectory *ftpGetDirLocal(const gchar *dir);

/* Used to create ftpFile structures from file names. Returns NULL on fail. */
/* If dir == NULL the current directory is used. Returns NULL on fail. */
extern ftpFile *ftpGetFileInfoRemote(ftpSite *site, const gchar *dir,
				     const gchar *name);
extern ftpFile *ftpGetFileInfoLocal(const gchar *dir, const gchar *name);

/* Refreshes dir (current directory if NULL), returns the new structure.
   All references to the old structure should be replaced!!
   Returns NULL on fail. */
extern gboolean ftpRefreshRemote(ftpSite *site, gchar *dir);
extern gboolean ftpRefreshLocal(gchar *dir);


/*****************************************************************************/
/* Transfer control functions. */

/*
 * If you want to resume a transfer, send the size of what's already
 * been transferred as resume, else send 0.
 *
 * local is only needed if you're uploading; otherwise you can saefely send
 * NULL.
 *
 * Returns 0 on fail, else number of bytes to be transferred.
 */
extern guint32 ftpStartTransfer(ftpSite *site, ftpFile *remote,
				ftpFile *local, guint32 resume,
				ftpDirection direction);

/* Reads n bytes from site into buf. Returns number of bytes read. */
extern gint ftpRead(ftpSite *site, gchar *buf, gint nbytes);

/* Writes n bytes from buf to site. Returns number of bytes written. */
extern gint ftpWrite(ftpSite *site, gchar *buf, gint nbytes);

/* Call to interrupt a transfer. */
extern gboolean ftpAbort(ftpSite *site);
/* Call only after receiving/writing all bytes from a transfer. */
extern gboolean ftpEndTransfer(ftpSite *site);

/*****************************************************************************/
/* Queue control functions. */

/* load/save the queue from/to fn */
extern gboolean            ftpQueueSave(gchar *fn);
extern gboolean            ftpQueueLoad(gchar *fn);

extern gint                ftpQueueGetSize(void);
extern gboolean            ftpQueueGoing(void);
extern const ftpQueueItem *ftpQueueGetNthItem(const gint n);

extern ftpQueueItem       *ftpQueueAdd(ftpSite *site, ftpFile *remotefile,
				       ftpFile *localfile,
				       ftpDirection direction);
extern void                ftpQueueRemove(ftpQueueItem *item);
extern gboolean            ftpQueueMoveUp(ftpQueueItem *item);
extern gboolean            ftpQueueMoveDown(ftpQueueItem *item);
extern void                ftpQueueClear(void);
extern void                ftpQueueStop(void);

/*
** The callback will be called on errors and as data is transferred. The
** callback will be passed a ftpQueueData structure (see above) so the
** calling program can know what the queue's doing. It is also passed
** userdata, which can be anything.
**
** The ftpLogCallback behaves exactly as the one you set for a site, but
** the queue doesn't set one for the sites it creates, it uses the one
** you pass here, as well as the data.
**
** current is the site the calling program is currently connected to.
** If reuseConnection is TRUE, this connection will be reused on any
** item on the same site (same login and port also).
*/
extern gboolean            ftpQueueGo(ftpQueueCallback callback,
				      gpointer userdata,
				      ftpLogCallback logfunc, gpointer command,
				      gpointer reply, gpointer logdata,
				      ftpSite *current);

/*****************************************************************************/
/* Error detection */

typedef enum {
	/* Connection errors */
	FTP_LOSTCONN,
	FTP_SOCKFAIL,
	FTP_FILESOCKFAIL,
	FTP_CONNECTED,
	FTP_NOTCONNECTED,

	/* Command errors */
	FTP_USERFAIL,
	FTP_PASSFAIL,
	FTP_TYPEFAIL,
	FTP_RLISTFAIL,
	FTP_LLISTFAIL,
	FTP_RCHDIRFAIL,
	FTP_LCHDIRFAIL,
	FTP_NOOPFAIL,

	/* Transfer errors */
	FTP_TRANSFERRING,
	FTP_NOTTRANSFERRING,
	FTP_TRANSFERFAIL,
	FTP_NOTDOWN,
	FTP_NOTUP,
	FTP_TLOSTCONN,
	FTP_RESTFAIL,
	FTP_ABORTFAIL,

	/* Queue errors */
	FTP_QGOING,
	FTP_QNOTQUEUED,
	FTP_QFIRST,
	FTP_QLAST,

	/* Miscellaneous errors */
	FTP_SYS,
} ftpError;

/* just like errno */
#ifdef HAVE_PTHREADS
extern pthread_key_t ftperrno_key;
#define ftperrno *( (ftpError *)pthread_getspecific(ftperrno_key) )
#else
extern ftpError ftperrno;
#endif

/* The largest message number in the errlist is ftp_nerr - 1. */
extern const gint ftp_nerr;
extern const gchar *ftp_errlist[];

/* just like strerror, except that the return value must be freed */
extern gchar *ftp_strerror(ftpError errnum);
extern void ftp_perror(gchar *s);

/*****************************************************************************/
/* Utility functions */

extern gboolean ftpEatSlashes(gchar *s, gint nbytes);
extern gchar *ftpParseEnv(gchar *in);

/* If link points to a file, this returns its size. It returns 0 for directory,
   and -1 on error. */
extern guint32 ftpRemoteLinkIsFile(ftpSite *site, const gchar *link);

/* Returns 1 if file exists, 0 if not, -1 on error */
extern gint ftpLocalFileExists(const gchar *fn);

/* Makes sure the given string doesn't contain characters that could cause
   security holes.

   Returns 1 if invalid characters were found (in this case, the string
   has been terminated at the first illegal character), else 0. */
extern char HTCleanTelnetString(char *str);

/* According to HTParse.h, it returns "a string which might be the old one
   or a new one". I supose this might be a memory leak, I don't know. I
   looked at the code and didn't see any memory allocation. It looks to me
   like url contains, the string you want, so I ignore the return value. */
extern char *HTSimplify(char **url);

extern void HTScan(char *name, HTURI *parts);

/*****************************************************************************/
/* Socket stuff */

/* returns socket descriptor, -1 on error */
extern int OpenSocket(char *host, int port);

/* returns pointer to a FILE stream, NULL on error */
extern FILE *FileSocket(int skt, char *mode);

#endif
